@@ -1,5 +1,5 @@
 # test_patch.py -- tests for patch.py
-# Copyright (C) 2010 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2010 Jelmer Vernooij <jelmer@jelmer.uk>
 #
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
@@ -32,6 +32,7 @@ from dulwich.object_store import (
     MemoryObjectStore,
     )
 from dulwich.patch import (
+    get_summary,
     git_am_patch_split,
     write_blob_diff,
     write_commit_patch,
@@ -166,7 +167,7 @@ Date: Thu, 15 Apr 2010 15:40:28 +0200
 Subject:  [Dulwich-users] [PATCH] Added unit tests for
  dulwich.object_store.tree_lookup_path.
 
-From: Jelmer Vernooy <jelmer@debian.org>
+From: Jelmer Vernooij <jelmer@debian.org>
 
 * dulwich/tests/test_object_store.py
   (TreeLookupPathTests): This test case contains a few tests that ensure the
@@ -180,7 +181,7 @@ From: Jelmer Vernooy <jelmer@debian.org>
 1.7.0.4
 """  # noqa: W291
         c, diff, version = git_am_patch_split(BytesIO(text), "utf-8")
-        self.assertEqual(b"Jelmer Vernooy <jelmer@debian.org>", c.author)
+        self.assertEqual(b"Jelmer Vernooij <jelmer@debian.org>", c.author)
         self.assertEqual(b'''\
 Added unit tests for dulwich.object_store.tree_lookup_path.
 
@@ -197,7 +198,7 @@ Date: Thu, 15 Apr 2010 15:40:28 +0200
 Subject:  [Dulwich-users] [PATCH] Added unit tests for
  dulwich.object_store.tree_lookup_path.
 
-From: Jelmer Vernooy <jelmer@debian.org>
+From: Jelmer Vernooij <jelmer@debian.org>
 
 ---
  pixmaps/prey.ico |  Bin 9662 -> 9662 bytes
@@ -276,9 +277,9 @@ class DiffTests(TestCase):
             f, (None, None, None),
             (b"bar.txt", 0o644, Blob.from_string(b"new\nsame\n")))
         self.assertEqual([
-             b'diff --git /dev/null b/bar.txt',
-             b'new mode 644',
-             b'index 0000000..a116b51 644',
+             b'diff --git a/bar.txt b/bar.txt',
+             b'new file mode 644',
+             b'index 0000000..a116b51',
              b'--- /dev/null',
              b'+++ b/bar.txt',
              b'@@ -0,0 +1,2 @@',
@@ -292,8 +293,8 @@ class DiffTests(TestCase):
             f, (b"bar.txt", 0o644, Blob.from_string(b"new\nsame\n")),
             (None, None, None))
         self.assertEqual([
-            b'diff --git a/bar.txt /dev/null',
-            b'deleted mode 644',
+            b'diff --git a/bar.txt b/bar.txt',
+            b'deleted file mode 644',
             b'index a116b51..0000000',
             b'--- a/bar.txt',
             b'+++ /dev/null',
@@ -322,9 +323,9 @@ class DiffTests(TestCase):
             tree1, tree2, added, removed, changed1, changed2, unchanged]])
         write_tree_diff(f, store, tree1.id, tree2.id)
         self.assertEqual([
-            b'diff --git /dev/null b/added.txt',
-            b'new mode 644',
-            b'index 0000000..76d4bb8 644',
+            b'diff --git a/added.txt b/added.txt',
+            b'new file mode 644',
+            b'index 0000000..76d4bb8',
             b'--- /dev/null',
             b'+++ b/added.txt',
             b'@@ -0,0 +1 @@',
@@ -337,8 +338,8 @@ class DiffTests(TestCase):
             b' unchanged',
             b'-removed',
             b'+added',
-            b'diff --git a/removed.txt /dev/null',
-            b'deleted mode 644',
+            b'diff --git a/removed.txt b/removed.txt',
+            b'deleted file mode 644',
             b'index 2c3f0b3..0000000',
             b'--- a/removed.txt',
             b'+++ /dev/null',
@@ -363,8 +364,8 @@ class DiffTests(TestCase):
             b'--- a/asubmodule',
             b'+++ b/asubmodule',
             b'@@ -1 +1 @@',
-            b'-Submodule commit 06d0bdd9e2e20377b3180e4986b14c8549b393e4',
-            b'+Submodule commit cc975646af69f279396d4d5e1379ac6af80ee637',
+            b'-Subproject commit 06d0bdd9e2e20377b3180e4986b14c8549b393e4',
+            b'+Subproject commit cc975646af69f279396d4d5e1379ac6af80ee637',
             ], f.getvalue().splitlines())
 
     def test_object_diff_blob(self):
@@ -394,9 +395,9 @@ class DiffTests(TestCase):
         write_object_diff(f, store, (None, None, None),
                                     (b"bar.txt", 0o644, b2.id))
         self.assertEqual([
-             b'diff --git /dev/null b/bar.txt',
-             b'new mode 644',
-             b'index 0000000..a116b51 644',
+             b'diff --git a/bar.txt b/bar.txt',
+             b'new file mode 644',
+             b'index 0000000..a116b51',
              b'--- /dev/null',
              b'+++ b/bar.txt',
              b'@@ -0,0 +1,2 @@',
@@ -412,8 +413,8 @@ class DiffTests(TestCase):
         write_object_diff(f, store, (b"bar.txt", 0o644, b1.id),
                                     (None, None, None))
         self.assertEqual([
-            b'diff --git a/bar.txt /dev/null',
-            b'deleted mode 644',
+            b'diff --git a/bar.txt b/bar.txt',
+            b'deleted file mode 644',
             b'index a116b51..0000000',
             b'--- a/bar.txt',
             b'+++ /dev/null',
@@ -492,9 +493,9 @@ class DiffTests(TestCase):
         write_object_diff(f, store, (None, None, None),
                                     (b'bar.png', 0o644, b2.id))
         self.assertEqual([
-            b'diff --git /dev/null b/bar.png',
-            b'new mode 644',
-            b'index 0000000..06364b7 644',
+            b'diff --git a/bar.png b/bar.png',
+            b'new file mode 644',
+            b'index 0000000..06364b7',
             b'Binary files /dev/null and b/bar.png differ'
             ], f.getvalue().splitlines())
 
@@ -510,8 +511,8 @@ class DiffTests(TestCase):
         write_object_diff(f, store, (b'foo.png', 0o644, b1.id),
                                     (None, None, None))
         self.assertEqual([
-            b'diff --git a/foo.png /dev/null',
-            b'deleted mode 644',
+            b'diff --git a/foo.png b/foo.png',
+            b'deleted file mode 644',
             b'index f73e47d..0000000',
             b'Binary files a/foo.png and /dev/null differ'
             ], f.getvalue().splitlines())
@@ -527,13 +528,25 @@ class DiffTests(TestCase):
                 b"06d0bdd9e2e20377b3180e4986b14c8549b393e4"))
         self.assertEqual([
             b'diff --git a/bar.txt b/bar.txt',
-            b'old mode 644',
-            b'new mode 160000',
+            b'old file mode 644',
+            b'new file mode 160000',
             b'index a116b51..06d0bdd 160000',
             b'--- a/bar.txt',
             b'+++ b/bar.txt',
             b'@@ -1,2 +1 @@',
             b'-new',
             b'-same',
-            b'+Submodule commit 06d0bdd9e2e20377b3180e4986b14c8549b393e4',
+            b'+Subproject commit 06d0bdd9e2e20377b3180e4986b14c8549b393e4',
             ], f.getvalue().splitlines())
+
+
+class GetSummaryTests(TestCase):
+
+    def test_simple(self):
+        c = Commit()
+        c.committer = c.author = b"Jelmer <jelmer@samba.org>"
+        c.commit_time = c.author_time = 1271350201
+        c.commit_timezone = c.author_timezone = 0
+        c.message = b"This is the first line\nAnd this is the second line.\n"
+        c.tree = Tree().id
+        self.assertEqual('This-is-the-first-line', get_summary(c))
